@@ -1,22 +1,25 @@
-"""Generators for classes of graphs used in studying social networks."""
-import itertools
-import math
-import random
-import networkx as nx
-#    Copyright(C) 2011, 2015 by
+#    Copyright(C) 2011, 2015, 2018 by
 #    Ben Edwards <bedwards@cs.unm.edu>
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Konstantinos Karakatsanis <dinoskarakas@gmail.com>
 #    All rights reserved.
 #    BSD license.
-__author__ = """\n""".join(['Ben Edwards (bedwards@cs.unm.edu)',
-                            'Aric Hagberg (hagberg@lanl.gov)',
-                            'Konstantinos Karakatsanis '
-                            '<dinoskarakas@gmail.com>'])
+#
+# Authors:  Ben Edwards (bedwards@cs.unm.edu)
+#           Aric Hagberg (hagberg@lanl.gov)
+#           Konstantinos Karakatsanis (dinoskarakas@gmail.com)
+#           Jean-Gabriel Young (jean.gabriel.young@gmail.com)
+"""Generators for classes of graphs used in studying social networks."""
+from __future__ import division
+import itertools
+import math
+import networkx as nx
+from networkx.utils import py_random_state
+
 __all__ = ['caveman_graph', 'connected_caveman_graph',
            'relaxed_caveman_graph', 'random_partition_graph',
            'planted_partition_graph', 'gaussian_random_partition_graph',
-           'ring_of_cliques', 'windmill_graph']
+           'ring_of_cliques', 'windmill_graph', 'stochastic_block_model']
 
 
 def caveman_graph(l, k):
@@ -108,6 +111,7 @@ def connected_caveman_graph(l, k):
     return G
 
 
+@py_random_state(3)
 def relaxed_caveman_graph(l, k, p, seed=None):
     """Return a relaxed caveman graph.
 
@@ -122,8 +126,9 @@ def relaxed_caveman_graph(l, k, p, seed=None):
       Size of cliques
     p : float
       Probabilty of rewiring each edge.
-    seed : int,optional
-      Seed for random number generator(default=None)
+    seed : integer, random_state, or None (default)
+        Indicator of random number generation state.
+        See :ref:`Randomness<randomness>`.
 
     Returns
     -------
@@ -145,13 +150,11 @@ def relaxed_caveman_graph(l, k, p, seed=None):
        Physics Reports Volume 486, Issues 3-5, February 2010, Pages 75-174.
        https://arxiv.org/abs/0906.0612
     """
-    if seed is not None:
-        random.seed(seed)
     G = nx.caveman_graph(l, k)
     nodes = list(G)
     for (u, v) in G.edges():
-        if random.random() < p:  # rewire the edge
-            x = random.choice(nodes)
+        if seed.random() < p:  # rewire the edge
+            x = seed.choice(nodes)
             if G.has_edge(u, x):
                 continue
             G.remove_edge(u, v)
@@ -159,6 +162,7 @@ def relaxed_caveman_graph(l, k, p, seed=None):
     return G
 
 
+@py_random_state(3)
 def random_partition_graph(sizes, p_in, p_out, seed=None, directed=False):
     """Return the random partition graph with a partition of sizes.
 
@@ -177,8 +181,9 @@ def random_partition_graph(sizes, p_in, p_out, seed=None, directed=False):
       probability of edges between groups
     directed : boolean optional, default=False
       Whether to create a directed graph
-    seed : int optional, default None
-      A seed for the random number generator
+    seed : integer, random_state, or None (default)
+        Indicator of random number generation state.
+        See :ref:`Randomness<randomness>`.
 
     Returns
     -------
@@ -213,73 +218,23 @@ def random_partition_graph(sizes, p_in, p_out, seed=None, directed=False):
     """
     # Use geometric method for O(n+m) complexity algorithm
     # partition = nx.community_sets(nx.get_node_attributes(G, 'affiliation'))
-    if seed is not None:
-        random.seed(seed)
     if not 0.0 <= p_in <= 1.0:
         raise nx.NetworkXError("p_in must be in [0,1]")
     if not 0.0 <= p_out <= 1.0:
         raise nx.NetworkXError("p_out must be in [0,1]")
 
-    if directed:
-        G = nx.DiGraph()
-    else:
-        G = nx.Graph()
-    G.graph['partition'] = []
-    n = sum(sizes)
-    G.add_nodes_from(range(n))
-    # start with len(sizes) groups of gnp random graphs with parameter p_in
-    # graphs are unioned together with node labels starting at
-    # 0, sizes[0], sizes[0]+sizes[1], ...
-    next_group = {}  # maps node key (int) to first node in next group
-    start = 0
-    group = 0
-    for n in sizes:
-        edges = ((u + start, v + start)
-                 for u, v in
-                 nx.fast_gnp_random_graph(n, p_in, directed=directed).edges())
-        G.add_edges_from(edges)
-        next_group.update(dict.fromkeys(range(start, start + n), start + n))
-        G.graph['partition'].append(set(range(start, start + n)))
-        group += 1
-        start += n
-    # handle edge cases
-    if p_out == 0:
-        return G
-    if p_out == 1:
-        for n in next_group:
-            targets = range(next_group[n], len(G))
-            G.add_edges_from(zip([n] * len(targets), targets))
-            if directed:
-                G.add_edges_from(zip(targets, [n] * len(targets)))
-        return G
-    # connect each node in group randomly with the nodes not in group
-    # use geometric method like fast_gnp_random_graph()
-    lp = math.log(1.0 - p_out)
-    n = len(G)
-    if directed:
-        for u in range(n):
-            v = 0
-            while v < n:
-                lr = math.log(1.0 - random.random())
-                v += int(lr / lp)
-                # skip over nodes in the same group as v, including self loops
-                if next_group.get(v, n) == next_group[u]:
-                    v = next_group[u]
-                if v < n:
-                    G.add_edge(u, v)
-                    v += 1
-    else:
-        for u in range(n - 1):
-            v = next_group[u]  # start with next node not in this group
-            while v < n:
-                lr = math.log(1.0 - random.random())
-                v += int(lr / lp)
-                if v < n:
-                    G.add_edge(u, v)
-                    v += 1
-    return G
+    # create connection matrix
+    num_blocks = len(sizes)
+    p = [[p_out for s in range(num_blocks)] for r in range(num_blocks)]
+    for r in range(num_blocks):
+        p[r][r] = p_in
+
+    return stochastic_block_model(sizes, p, nodelist=None, seed=seed,
+                                  directed=directed, selfloops=False,
+                                  sparse=True)
 
 
+@py_random_state(4)
 def planted_partition_graph(l, k, p_in, p_out, seed=None, directed=False):
     """Return the planted l-partition graph.
 
@@ -298,8 +253,9 @@ def planted_partition_graph(l, k, p_in, p_out, seed=None, directed=False):
       probability of connecting vertices within a group
     p_out : float
       probability of connected vertices between groups
-    seed : int,optional
-      Seed for random number generator(default=None)
+    seed : integer, random_state, or None (default)
+        Indicator of random number generation state.
+        See :ref:`Randomness<randomness>`.
     directed : bool,optional (default=False)
       If True return a directed graph
 
@@ -315,7 +271,7 @@ def planted_partition_graph(l, k, p_in, p_out, seed=None, directed=False):
 
     Examples
     --------
-    >>> G = nx.planted_partition_graph(4, 3, 0.5, 0.1,seed=42)
+    >>> G = nx.planted_partition_graph(4, 3, 0.5, 0.1, seed=42)
 
     See Also
     --------
@@ -333,6 +289,7 @@ def planted_partition_graph(l, k, p_in, p_out, seed=None, directed=False):
     return random_partition_graph([k] * l, p_in, p_out, seed, directed)
 
 
+@py_random_state(6)
 def gaussian_random_partition_graph(n, s, v, p_in, p_out, directed=False,
                                     seed=None):
     """Generate a Gaussian random partition graph.
@@ -356,8 +313,9 @@ def gaussian_random_partition_graph(n, s, v, p_in, p_out, directed=False,
       Probability of inter cluster connection.
     directed : boolean, optional default=False
       Whether to create a directed graph or not
-    seed : int
-      Seed value for random number generator
+    seed : integer, random_state, or None (default)
+        Indicator of random number generation state.
+        See :ref:`Randomness<randomness>`.
 
     Returns
     -------
@@ -397,7 +355,7 @@ def gaussian_random_partition_graph(n, s, v, p_in, p_out, directed=False,
     assigned = 0
     sizes = []
     while True:
-        size = int(random.normalvariate(s, float(s) / v + 0.5))
+        size = int(seed.gauss(s, float(s) / v + 0.5))
         if size < 1:  # how to handle 0 or negative sizes?
             continue
         if assigned + size >= n:
@@ -417,14 +375,14 @@ def ring_of_cliques(num_cliques, clique_size):
     Parameters
     ----------
     num_cliques : int
-      Number of cliques
+        Number of cliques
     clique_size : int
-      Size of cliques
+        Size of cliques
 
     Returns
     -------
     G : NetworkX Graph
-      ring of cliques graph
+        ring of cliques graph
 
     Raises
     ------
@@ -474,14 +432,14 @@ def windmill_graph(n, k):
     Parameters
     ----------
     n : int
-      Number of cliques
+        Number of cliques
     k : int
-      Size of cliques
+        Size of cliques
 
     Returns
     -------
     G : NetworkX Graph
-      windmill graph with n cliques of size k
+        windmill graph with n cliques of size k
 
     Raises
     ------
@@ -510,3 +468,171 @@ def windmill_graph(n, k):
                                                for _ in range(n - 1))))
     G.add_edges_from((0, i) for i in range(k, G.number_of_nodes()))
     return G
+
+
+@py_random_state(3)
+def stochastic_block_model(sizes, p, nodelist=None, seed=None,
+                           directed=False, selfloops=False, sparse=True):
+    """Return a stochastic block model graph.
+
+    This model partitions the nodes in blocks of arbitrary sizes, and places
+    edges between pairs of nodes independently, with a probability that depends
+    on the blocks.
+
+    Parameters
+    ----------
+    sizes : list of ints
+        Sizes of blocks
+    p : list of list of floats
+        Element (r,s) gives the density of edges going from the nodes
+        of group r to nodes of group s.
+        p must match the number of groups (len(sizes) == len(p)),
+        and it must be symmetric if the graph is undirected.
+    nodelist : list, optional
+        The block tags are assigned according to the node identifiers
+        in nodelist. If nodelist is None, then the ordering is the
+        range [0,sum(sizes)-1].
+    seed : integer, random_state, or None (default)
+        Indicator of random number generation state.
+        See :ref:`Randomness<randomness>`.
+    directed : boolean optional, default=False
+        Whether to create a directed graph or not.
+    selfloops : boolean optional, default=False
+        Whether to include self-loops or not.
+    sparse: boolean optional, default=True
+        Use the sparse heuristic to speed up the generator.
+
+    Returns
+    -------
+    g : NetworkX Graph or DiGraph
+        Stochastic block model graph of size sum(sizes)
+
+    Raises
+    ------
+    NetworkXError
+      If probabilities are not in [0,1].
+      If the probability matrix is not square (directed case).
+      If the probability matrix is not symmetric (undirected case).
+      If the sizes list does not match nodelist or the probability matrix.
+      If nodelist contains duplicate.
+
+    Examples
+    --------
+    >>> sizes = [75, 75, 300]
+    >>> probs = [[0.25, 0.05, 0.02],
+    ...          [0.05, 0.35, 0.07],
+    ...          [0.02, 0.07, 0.40]]
+    >>> g = nx.stochastic_block_model(sizes, probs, seed=0)
+    >>> len(g)
+    450
+    >>> H = nx.quotient_graph(g, g.graph['partition'], relabel=True)
+    >>> for v in H.nodes(data=True):
+    ...     print(round(v[1]['density'], 3))
+    ...
+    0.245
+    0.348
+    0.405
+    >>> for v in H.edges(data=True):
+    ...     print(round(1.0 * v[2]['weight'] / (sizes[v[0]] * sizes[v[1]]), 3))
+    ...
+    0.051
+    0.022
+    0.07
+
+    See Also
+    --------
+    random_partition_graph
+    planted_partition_graph
+    gaussian_random_partition_graph
+    gnp_random_graph
+
+    References
+    ----------
+    .. [1] Holland, P. W., Laskey, K. B., & Leinhardt, S.,
+           "Stochastic blockmodels: First steps",
+           Social networks, 5(2), 109-137, 1983.
+    """
+    # Check if dimensions match
+    if len(sizes) != len(p):
+        raise nx.NetworkXException("'sizes' and 'p' do not match.")
+    # Check for probability symmetry (undirected) and shape (directed)
+    for row in p:
+        if len(p) != len(row):
+            raise nx.NetworkXException("'p' must be a square matrix.")
+    if not directed:
+        p_transpose = [list(i) for i in zip(*p)]
+        for i in zip(p, p_transpose):
+            for j in zip(i[0], i[1]):
+                if abs(j[0] - j[1]) > 1e-08:
+                    raise nx.NetworkXException("'p' must be symmetric.")
+    # Check for probability range
+    for row in p:
+        for prob in row:
+            if prob < 0 or prob > 1:
+                raise nx.NetworkXException("Entries of 'p' not in [0,1].")
+    # Check for nodelist consistency
+    if nodelist is not None:
+        if len(nodelist) != sum(sizes):
+            raise nx.NetworkXException("'nodelist' and 'sizes' do not match.")
+        if len(nodelist) != len(set(nodelist)):
+            raise nx.NetworkXException("nodelist contains duplicate.")
+    else:
+        nodelist = range(0, sum(sizes))
+
+    # Setup the graph conditionally to the directed switch.
+    block_range = range(len(sizes))
+    if directed:
+        g = nx.DiGraph()
+        block_iter = itertools.product(block_range, block_range)
+    else:
+        g = nx.Graph()
+        block_iter = itertools.combinations_with_replacement(block_range, 2)
+    # Split nodelist in a partition (list of sets).
+    size_cumsum = [sum(sizes[0:x]) for x in range(0, len(sizes) + 1)]
+    g.graph['partition'] = [set(nodelist[size_cumsum[x]:size_cumsum[x + 1]])
+                            for x in range(0, len(size_cumsum) - 1)]
+    # Setup nodes and graph name
+    for block_id, nodes in enumerate(g.graph['partition']):
+        for node in nodes:
+            g.add_node(node, block=block_id)
+
+    g.name = "stochastic_block_model"
+
+    # Test for edge existence
+    parts = g.graph['partition']
+    for i, j in block_iter:
+        if i == j:
+            if directed:
+                if selfloops:
+                    edges = itertools.product(parts[i], parts[i])
+                else:
+                    edges = itertools.permutations(parts[i], 2)
+            else:
+                edges = itertools.combinations(parts[i], 2)
+                if selfloops:
+                    edges = itertools.chain(edges, zip(parts[i], parts[i]))
+            for e in edges:
+                if seed.random() < p[i][j]:
+                    g.add_edge(*e)
+        else:
+            edges = itertools.product(parts[i], parts[j])
+        if sparse:
+            if p[i][j] == 1:  # Test edges cases p_ij = 0 or 1
+                for e in edges:
+                    g.add_edge(*e)
+            elif p[i][j] > 0:
+                while True:
+                    try:
+                        logrand = math.log(seed.random())
+                        skip = math.floor(logrand / math.log(1 - p[i][j]))
+                        # consume "skip" edges
+                        next(itertools.islice(edges, skip, skip), None)
+                        e = next(edges)
+                        g.add_edge(*e)  # __safe
+                    except StopIteration:
+                        break
+        else:
+            for e in edges:
+                if seed.random() < p[i][j]:
+                    g.add_edge(*e)  # __safe
+    return g
