@@ -7,18 +7,18 @@ Do not add this file to the repository.
 
 import datetime
 
-version = '2.1'
-date = 'Wed Aug  8 06:08:15 2018'
+version = '2.2'
+date = 'Thu Feb 20 18:52:15 2020'
 
 # Was NetworkX built from a development version? If so, remember that the major
 # and minor versions reference the "target" (rather than "current") release.
 dev = False
 
 # Format: (name, major, min, revision)
-version_info = ('networkx', '2', '1', None)
+version_info = ('networkx', '2', '2', None)
 
 # Format: a 'datetime.datetime' instance
-date_info = datetime.datetime(2018, 8, 8, 6, 8, 15)
+date_info = datetime.datetime(2020, 2, 20, 18, 52, 15)
 
 # Format: (vcs, vcs_tuple)
 vcs_info = (None, (None, None))
